@@ -385,9 +385,19 @@ sw_size_allocate(GtkWidget *widget, GtkAllocation *alloc, gpointer data)
 	GtkGS *view = GTK_GS(data);
 	guint border = gtk_container_get_border_width(GTK_CONTAINER(widget));
 	gint w, h;
+	gint scrollbar_spacing;
+	GtkRequisition req;
 
-	w = alloc->width - 2*border;
-	h = alloc->height - 2*border;
+	gtk_widget_style_get (widget,
+						  "scrollbar_spacing", &scrollbar_spacing,
+						  NULL);
+	w = alloc->width - 2*border - 2*widget->style->xthickness - 
+		2*scrollbar_spacing;
+	if(NULL != GTK_SCROLLED_WINDOW(widget)->vscrollbar) {
+		gtk_widget_size_request(GTK_SCROLLED_WINDOW(widget)->vscrollbar, &req);
+		w -= req.width;
+	}
+	h = alloc->height - 2*border - 2*widget->style->ythickness;
 	w = MAX(w, 0);
 	h = MAX(h, 0);
 	gtk_gs_set_available_size(view, w, h);
@@ -888,8 +898,8 @@ ps_view_property_changed_handler(BonoboListener    *listener,
 {
         GgvControl *control = GGV_CONTROL(data);
 
-        if(!g_ascii_strcasecmp(event_name, "Bonobo/Property:change:page")) {
-			set_command_items_sensitivity(control, BONOBO_ARG_GET_INT(any) != -1);
+        if(!g_ascii_strcasecmp(event_name, "Bonobo/Property:change:page_count")) {
+			set_command_items_sensitivity(control, BONOBO_ARG_GET_LONG(any) > 0);
         }
 }
 
@@ -1001,18 +1011,12 @@ ggv_control_construct (GgvControl *control, GgvPostScriptView *ps_view)
 
 	bonobo_control_set_properties (BONOBO_CONTROL(control),
 								   BONOBO_OBJREF(property_bag), NULL);
-	mask = "Bonobo/Property:change:page";
+	mask = "Bonobo/Property:change:page_count";
 	bonobo_event_source_client_add_listener(BONOBO_OBJREF(property_bag),
 											(BonoboListenerCallbackFn)ps_view_property_changed_handler,
 											mask, NULL, control);
 	bonobo_object_unref (BONOBO_OBJECT (property_bag));
 
-	property_control =
-		ggv_postscript_view_get_property_control (control->priv->ps_view);
-	bonobo_object_add_interface (BONOBO_OBJECT (control),
-								 BONOBO_OBJECT (property_control));
-	bonobo_object_unref(BONOBO_OBJECT(property_control));
-
 	control->priv->uic =
 		bonobo_control_get_ui_component (BONOBO_CONTROL (control));
 
