@@ -7,9 +7,12 @@
  * Copyright (c) 2001, 2002 Free Software Foundation
  */
 
+#include <strings.h>
 #include <config.h>
 #include <stdio.h>
+#include <gtk/gtkfilechooserdialog.h>
 #include <gtk/gtksignal.h>
+#include <gtk/gtkstock.h>
 #include <gtk/gtkmarshal.h>
 #include <gtk/gtktypeutils.h>
 #include <gconf/gconf-client.h>
@@ -18,13 +21,13 @@
 
 #include <ggv-postscript-view.h>
 #include <ggv-control.h>
+#include <ggv-document.h>
 
 #include <gtkgs.h>
 #include <ps.h>
 #include <gsdefaults.h>
 #include <ggvutils.h>
 #include <ggv-msg-window.h>
-#include <ggv-file-sel.h>
 #include <ggv-prefs.h>
 #include <ggv-prefs-ui.h>
 #include <ggv-sidebar.h>
@@ -50,13 +53,16 @@ struct _GgvPostScriptViewPrivate {
 	GgvSidebar *sidebar;
 
 	BonoboPropertyBag     *property_bag;
-	BonoboPropertyControl *property_control;
 
 	BonoboUIComponent     *uic, *popup_uic;
 
 	gboolean               zoom_fit;
 
 	gboolean pane_auto_jump, page_flip;
+
+	BonoboControl *owner;
+
+	GdkCursor *pan_cursor, *win_cursor;
 };
 
 struct _GgvPostScriptViewClassPrivate {
@@ -70,7 +76,6 @@ enum {
 	PROP_DEFAULT_SIZE,
 	PROP_OVERRIDE_SIZE,
 	PROP_RESPECT_EOF,
-	PROP_WATCH_FILE,
 	PROP_WIDTH,
 	PROP_HEIGHT,
 	PROP_ORIENTATION,
@@ -81,12 +86,6 @@ enum {
 	PROP_STATUS
 };
 
-enum {
-	PROP_CONTROL_TITLE,
-};
-
-static GdkCursor *pan_cursor = NULL;
-
 static BonoboObjectClass *ggv_postscript_view_parent_class;
 
 static const gchar *orientation_paths[] = {
@@ -218,12 +217,12 @@ notify_page_count_change(GgvPostScriptView *ps_view)
 	BonoboArg *arg;
 
 	set_page_items_sensitivity(ps_view);
-	set_file_items_sensitivity(ps_view, ggv_postscript_view_get_page_count(ps_view) > 0);
+	set_file_items_sensitivity(ps_view, ggv_document_get_page_count(GGV_DOCUMENT(ps_view)) > 0);
 	if(ps_view->priv->sidebar)
 		ggv_sidebar_create_page_list(ps_view->priv->sidebar);
 
 	arg = bonobo_arg_new(TC_CORBA_long);
-	BONOBO_ARG_SET_LONG(arg, ggv_postscript_view_get_page_count(ps_view));
+	BONOBO_ARG_SET_LONG(arg, ggv_document_get_page_count(GGV_DOCUMENT(ps_view)));
 	bonobo_event_source_notify_listeners(ps_view->priv->property_bag->es,
 										 "Bonobo/Property:change:page_count",
 										 arg, NULL);
@@ -250,15 +249,17 @@ notify_page_change(GgvPostScriptView *ps_view)
 	BonoboArg *arg;
 
 	set_page_items_sensitivity(ps_view);
-	ggv_sidebar_page_changed(ps_view->priv->sidebar,
-							 gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs)));
+	if(gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs)) >= 0) {
+		ggv_sidebar_page_changed(ps_view->priv->sidebar,
+								 gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs)));
 
-	arg = bonobo_arg_new(TC_CORBA_long);
-	*(CORBA_long *)arg->_value = gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs));
-	bonobo_event_source_notify_listeners(ps_view->priv->property_bag->es,
-										 "Bonobo/Property:change:page",
-										 arg, NULL);
-	bonobo_arg_release(arg);
+		arg = bonobo_arg_new(TC_CORBA_long);
+		*(CORBA_long *)arg->_value = gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs));
+		bonobo_event_source_notify_listeners(ps_view->priv->property_bag->es,
+											 "Bonobo/Property:change:page",
+											 arg, NULL);
+		bonobo_arg_release(arg);
+	}
 }
 
 static void
@@ -306,9 +307,10 @@ ggv_postscript_view_get_current_page(GgvPostScriptView *ps_view)
 	return gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs));
 }
 
-gint
-ggv_postscript_view_get_page_count(GgvPostScriptView *ps_view)
+static gint
+ggv_postscript_view_get_page_count(GgvDocument *doc)
 {
+    GgvPostScriptView *ps_view = GGV_POSTSCRIPT_VIEW(doc);
 	return gtk_gs_get_page_count(GTK_GS(ps_view->priv->gs));
 }
 
@@ -322,7 +324,7 @@ load_ps(GgvPostScriptView *ps_view, const gchar *fname)
 	ggv_postscript_view_goto_page(ps_view, 0);
 	notify_page_change(ps_view);
 	ggv_sidebar_update_coordinates(ps_view->priv->sidebar, 0.0, 0.0);
-	set_file_items_sensitivity(ps_view, ggv_postscript_view_get_page_count(ps_view) > 0);
+	set_file_items_sensitivity(ps_view, ggv_document_get_page_count(GGV_DOCUMENT(ps_view)) > 0);
 	notify_status_change(ps_view);
 }
 
@@ -379,9 +381,12 @@ load_ps_from_stream (BonoboPersistStream *ps,
 
 	ps_view = GGV_POSTSCRIPT_VIEW(data);
 
+	if(ps_view->priv->uri) {
+		g_free(ps_view->priv->uri);
+		ps_view->priv->uri = NULL;
+	}
 	ps_view_clean_tmp_file(ps_view);
 	tmpfile = ps_view_get_tmp_file(ps_view);
-
 	do {
 		Bonobo_Stream_read (stream, 32768, &buffer, ev);
 		if (ev->_major != CORBA_NO_EXCEPTION)
@@ -422,22 +427,20 @@ view_button_press_cb(GtkWidget *widget, GdkEventButton *event, gpointer data)
 
 	if(event->button == 1 && !ps_view->priv->pan) {
 		gint wx = 0, wy = 0;
-			
 		gtk_widget_grab_focus(ps_view->priv->gs);
-
 		gdk_window_get_pointer(widget->window, &wx, &wy, NULL);
 			
 		gtk_gs_start_scroll(GTK_GS(ps_view->priv->gs));
 
 		ps_view->priv->pan = TRUE;
-		if(pan_cursor == NULL)
-			pan_cursor = cursor_get(widget->window, CURSOR_HAND_CLOSED);
-			
+		if(ps_view->priv->pan_cursor == NULL)
+			ps_view->priv->pan_cursor =
+				cursor_get(widget->window, CURSOR_HAND_CLOSED);
 		gtk_grab_add(widget);
 		gdk_pointer_grab(widget->window, FALSE,
 						 GDK_POINTER_MOTION_MASK |
 						 GDK_BUTTON_RELEASE_MASK, NULL,
-						 pan_cursor, GDK_CURRENT_TIME);
+						 ps_view->priv->pan_cursor, GDK_CURRENT_TIME);
 		ps_view->priv->prev_x = wx;
 		ps_view->priv->prev_y = wy;
 
@@ -528,6 +531,12 @@ load_ps_from_file (BonoboPersistFile *pf, const CORBA_char *filename,
 
 	ps_view_clean_tmp_file(ps_view);
 
+	uri_str = ps_view->priv->uri;
+	ps_view->priv->uri = g_strdup(filename);
+	if(NULL != uri_str) {
+		g_free(uri_str);
+		uri_str = NULL;
+	}
 	uri = gnome_vfs_uri_new(filename);
 	if(gnome_vfs_uri_is_local(uri)) {
 		uri_str = gnome_vfs_uri_to_string(uri, GNOME_VFS_URI_HIDE_NONE);
@@ -745,6 +754,11 @@ ggv_postscript_view_print(GgvPostScriptView *ps_view, gchar *ps)
 									 GTK_BUTTONS_OK,
 									 _("Unable to execute print command:\n%s"),
 									 ggv_print_cmd);
+		if(ps_view->priv->owner != NULL) {
+			bonobo_control_set_transient_for(ps_view->priv->owner,
+											 GTK_WINDOW(dlg), NULL);
+			gtk_window_set_destroy_with_parent(GTK_WINDOW(dlg), TRUE);
+		}
 		gtk_widget_show(dlg);
 		gtk_dialog_run(GTK_DIALOG(dlg));
 		gtk_widget_destroy(dlg);
@@ -787,18 +801,23 @@ verb_FileSaveMarked(BonoboUIComponent *uic, gpointer data, const char *cname)
 {
 	GgvPostScriptView *ps_view = GGV_POSTSCRIPT_VIEW(data);
 	gint *active_rows;
-	gchar *doc, *fname, *local_path = NULL;
+	gchar *doc, *fname = NULL, *local_path = NULL;
 
 	active_rows = ggv_sidebar_get_active_list(ps_view->priv->sidebar);
 	if(active_rows[0] == -1) {
 		GtkWidget *dlg;
 		
-		dlg = gtk_message_dialog_new(GTK_WINDOW(gtk_widget_get_toplevel(ps_view->priv->gs)),
-									 GTK_DIALOG_MODAL,
+		dlg = gtk_message_dialog_new(NULL,
+									 0,
 									 GTK_MESSAGE_QUESTION,
 									 GTK_BUTTONS_YES_NO,
 									 _("No pages have been marked.\n"
 									   "Do you want to save the whole document?"));
+		if(ps_view->priv->owner != NULL) {
+			bonobo_control_set_transient_for(ps_view->priv->owner,
+											 GTK_WINDOW(dlg), NULL);
+			gtk_window_set_destroy_with_parent(GTK_WINDOW(dlg), TRUE);
+		}
 		gtk_widget_show(dlg);
 		switch(gtk_dialog_run(GTK_DIALOG(dlg))) {
 		case GTK_RESPONSE_NO:
@@ -817,8 +836,30 @@ verb_FileSaveMarked(BonoboUIComponent *uic, gpointer data, const char *cname)
 	g_free(active_rows);
 
 	if(doc != NULL) {
-		fname = ggv_file_sel_request_uri(_("Select a file to save pages as"),
-										 ps_view->priv->save_path);
+		GtkWidget *chooser;
+
+		chooser = gtk_file_chooser_dialog_new(_("Select a file to save pages as"),
+						      GTK_WINDOW(gtk_widget_get_toplevel(ps_view->priv->gs)),
+						      GTK_FILE_CHOOSER_ACTION_SAVE,
+						      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+						      GTK_STOCK_SAVE, GTK_RESPONSE_OK,
+						      NULL);
+		if(ps_view->priv->owner != NULL) {
+			bonobo_control_set_transient_for(ps_view->priv->owner,
+											 GTK_WINDOW(chooser), NULL);
+			gtk_window_set_destroy_with_parent(GTK_WINDOW(chooser), TRUE);
+		}
+		gtk_dialog_set_default_response(GTK_DIALOG(chooser), GTK_RESPONSE_OK);
+		gtk_window_set_default_size(GTK_WINDOW(chooser), 600, 400);
+		if (ps_view->priv->save_path)
+			gtk_file_chooser_set_current_folder_uri(GTK_FILE_CHOOSER(chooser),
+						 		ps_view->priv->save_path);
+
+		if (gtk_dialog_run(GTK_DIALOG(chooser)) == GTK_RESPONSE_OK)
+			fname = gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(chooser));
+
+		gtk_widget_destroy(chooser);
+
 		if(fname) {
 			if((local_path = gnome_vfs_get_local_path_from_uri(fname)) &&
 			   ggv_file_length(local_path) >= 0) {
@@ -831,6 +872,11 @@ verb_FileSaveMarked(BonoboUIComponent *uic, gpointer data, const char *cname)
 											 GTK_BUTTONS_YES_NO,
 											 _("A file with this name already exists.\n"
 											   "Do you want to overwrite it?"));
+				if(ps_view->priv->owner != NULL) {
+					bonobo_control_set_transient_for(ps_view->priv->owner,
+													 GTK_WINDOW(dlg), NULL);
+					gtk_window_set_destroy_with_parent(GTK_WINDOW(dlg), TRUE);
+				}
 				gtk_widget_show(dlg);
 				switch(gtk_dialog_run(GTK_DIALOG(dlg))) {
 				case GTK_RESPONSE_NO:
@@ -870,6 +916,11 @@ verb_FilePrintMarked(BonoboUIComponent *uic, gpointer data, const char *cname)
 									 GTK_BUTTONS_YES_NO,
 									 _("No pages have been marked.\n"
 									   "Do you want to print the whole document?"));
+		if(ps_view->priv->owner != NULL) {
+			bonobo_control_set_transient_for(ps_view->priv->owner,
+											 GTK_WINDOW(dlg), NULL);
+			gtk_window_set_destroy_with_parent(GTK_WINDOW(dlg), TRUE);
+		}			
 		gtk_widget_show(dlg);
 		switch(gtk_dialog_run(GTK_DIALOG(dlg))) {
 		case GTK_RESPONSE_NO:
@@ -1020,19 +1071,21 @@ ggv_postscript_view_create_ui(GgvPostScriptView *ps_view)
 										 orientation_paths[i] + strlen("/commands/"),
 										 listener_Orientation_cb, ps_view);
 	set_file_items_sensitivity(ps_view,
-							   ggv_postscript_view_get_page_count(ps_view) > 0);
+							   ggv_document_get_page_count(GGV_DOCUMENT(ps_view)) > 0);
 	set_page_items_sensitivity(ps_view);
 	sync_orientation_items(ps_view);
 	sync_size_items(ps_view);
 }
 
-gchar **
-ggv_postscript_view_get_page_names(GgvPostScriptView *ps_view)
+static gchar **
+ggv_postscript_view_get_page_names(GgvDocument *doc)
 {
 	gchar **names = NULL;
 	GtkGS *gs;
 	gint i;
+	GgvPostScriptView *ps_view;
 
+	ps_view = GGV_POSTSCRIPT_VIEW(doc);
 	gs = GTK_GS(ps_view->priv->gs);
 	if(gs->loaded && gs->doc && gs->structured_doc) {
 		names = g_new0(gchar *, gs->doc->numpages + 1);
@@ -1063,6 +1116,8 @@ ggv_postscript_view_get_prop(BonoboPropertyBag *bag,
 		g_assert(arg->_type == TC_CORBA_long);
 
 		*(CORBA_long *)arg->_value = gtk_gs_get_current_page(GTK_GS(ps_view->priv->gs));
+		if((*(CORBA_long *)arg->_value) < 0)
+			*(CORBA_long *)arg->_value = 0;
 		break;
 	}
 	case PROP_PAGE_COUNT: {
@@ -1225,14 +1280,6 @@ ggv_postscript_view_get_prop(BonoboPropertyBag *bag,
 			CORBA_TRUE:CORBA_FALSE;
 		break;
 	}
-	case PROP_WATCH_FILE: {
-		g_assert(arg->_type == TC_CORBA_boolean);
-
-		*(CORBA_boolean *)arg->_value =
-			gtk_gs_get_watch_file(GTK_GS(ps_view->priv->gs))?
-			CORBA_TRUE:CORBA_FALSE;
-		break;
-	}
 	default:
 		g_assert_not_reached();
 	}
@@ -1327,11 +1374,6 @@ ggv_postscript_view_set_prop(BonoboPropertyBag *bag,
 								*(CORBA_boolean *)arg->_value);
 		 break;
 	}
-	case PROP_WATCH_FILE: {
-		 gtk_gs_set_watch_file(GTK_GS(ps_view->priv->gs),
-							   *(CORBA_boolean *)arg->_value);
-		 break;
-	}
 	default:
 		g_assert_not_reached();
 	}
@@ -1351,28 +1393,31 @@ ggv_postscript_view_get_object(BonoboItemContainer *item_container,
 	g_return_val_if_fail(ps_view != NULL, CORBA_OBJECT_NIL);
 	g_return_val_if_fail(GGV_IS_POSTSCRIPT_VIEW(ps_view), CORBA_OBJECT_NIL);
 
+#ifdef _DEBUG
 	g_message ("ggv_postscript_view_get_object: %d - %s",
 			   only_if_exists, item_name);
-
+#endif /* _DEBUG */
 	params = ggv_split_string (item_name, "!");
 	for (c = params; c; c = c->next) {
 		gchar *name = c->data;
 
 		if ((!strcmp (name, "control") || !strcmp (name, "embeddable"))
 		    && (object != NULL)) {
-			g_message ("ggv_postscript_view_get_object: "
+			g_warning ("ggv_postscript_view_get_object: "
 					   "can only return one kind of an Object");
 			continue;
 		}
 
-		if (!strcmp (name, "control"))
+		if (!strcmp (name, "control")) {
 			object = (BonoboObject *) ggv_control_new (ps_view);
-#if 0
+			ps_view->priv->owner = (BonoboControl *)object;
+		}
+#ifdef ENABLE_GGV_EMBEDDABLE
 		else if (!strcmp (item_name, "embeddable"))
 			object = (BonoboObject *) ggv_embeddable_new (image);
-#endif
+#endif /* ENABLE_GGV_EMBEDDABLE */
 		else
-			g_message ("ggv_postscript_view_get_object: "
+			g_warning ("ggv_postscript_view_get_object: "
 					   "unknown parameter `%s'",
 					   name);
 	}
@@ -1450,20 +1495,9 @@ ggv_postscript_view_get_property_bag(GgvPostScriptView *ps_view)
 	return ps_view->priv->property_bag;
 }
 
-BonoboPropertyControl *
-ggv_postscript_view_get_property_control(GgvPostScriptView *ps_view)
-{
-	g_return_val_if_fail(ps_view != NULL, NULL);
-	g_return_val_if_fail(GGV_IS_POSTSCRIPT_VIEW(ps_view), NULL);
-
-	bonobo_object_ref(BONOBO_OBJECT(ps_view->priv->property_control));
-
-	return ps_view->priv->property_control;
-}
-
 void
 ggv_postscript_view_set_ui_container(GgvPostScriptView *ps_view,
-				 Bonobo_UIContainer ui_container)
+									 Bonobo_UIContainer ui_container)
 {
 	g_return_if_fail(ps_view != NULL);
 	g_return_if_fail(GGV_IS_POSTSCRIPT_VIEW(ps_view));
@@ -1609,6 +1643,14 @@ ggv_postscript_view_destroy(BonoboObject *object)
 		bonobo_object_unref(BONOBO_OBJECT(ps_view->priv->sidebar));
 		ps_view->priv->sidebar = NULL;
 	}
+	if(ps_view->priv->pan_cursor) {
+		gdk_cursor_unref(ps_view->priv->pan_cursor);
+		ps_view->priv->pan_cursor = NULL;
+	}
+	if(ps_view->priv->win_cursor) {
+		gdk_cursor_unref(ps_view->priv->win_cursor);
+		ps_view->priv->win_cursor = NULL;
+	}
 	ps_view_clean_tmp_file(ps_view);
 
 	if (BONOBO_OBJECT_CLASS (ggv_postscript_view_parent_class)->destroy)
@@ -1627,10 +1669,10 @@ ggv_postscript_view_finalize(GObject *object)
 
 	ps_view_clean_tmp_file(ps_view);
 
-	if(ps_view->priv->save_path) {
+	if(ps_view->priv->save_path)
 		g_free(ps_view->priv->save_path);
-		ps_view->priv->save_path = NULL;
-	}
+	if(ps_view->priv->uri)
+		g_free(ps_view->priv->uri);
 
 	g_free(ps_view->priv);
 
@@ -1680,7 +1722,7 @@ impl_GNOME_GGV_PostScriptView_getPages(PortableServer_Servant servant,
 
 static void
 impl_GNOME_GGV_PostScriptView_reload(PortableServer_Servant servant,
-									   CORBA_Environment *ev)
+									 CORBA_Environment *ev)
 {
 	GgvPostScriptView *ps_view;
 
@@ -1729,81 +1771,53 @@ ggv_postscript_view_class_init(GgvPostScriptViewClass *klass)
 	ggv_prefs_load();
 }
 
+static void
+ggv_document_interface_init (GgvDocumentClass *klass, gpointer iface_data)
+{
+	klass->get_page_count = ggv_postscript_view_get_page_count;
+	klass->get_page_names = ggv_postscript_view_get_page_names;
+}
+
 static void
 ggv_postscript_view_init(GgvPostScriptView *ps_view)
 {
 	ps_view->priv = g_new0(GgvPostScriptViewPrivate, 1);
 }
 
-BONOBO_TYPE_FUNC_FULL(GgvPostScriptView, 
-					  GNOME_GGV_PostScriptView,
-					  BONOBO_TYPE_OBJECT,
-					  ggv_postscript_view);
-
-#if 0
-static void
-property_control_get_prop(BonoboPropertyBag *bag,
-						  BonoboArg         *arg,
-						  guint              arg_id,
-						  CORBA_Environment *ev,
-						  gpointer           user_data)
+GType
+ggv_postscript_view_get_type (void)
 {
-	switch(arg_id) {
-	case PROP_CONTROL_TITLE:
-		g_assert(arg->_type == BONOBO_ARG_STRING);
-		BONOBO_ARG_SET_STRING(arg, _("Display"));
-		break;
-	default:
-		g_assert_not_reached();
-	}
-}
-#endif
-
-static BonoboControl *
-property_control_get_cb(BonoboPropertyControl *property_control,
-						int page_number, void *closure)
-{
-	GgvPostScriptView *ps_view;
-
-	g_return_val_if_fail(closure != NULL, NULL);
-	g_return_val_if_fail(GGV_IS_POSTSCRIPT_VIEW(closure), NULL);
-	g_return_val_if_fail(page_number == 0, NULL);
-
-	ps_view = GGV_POSTSCRIPT_VIEW(closure);
-
-#if 0
-	container = eog_create_preferences_page(ps_view, page_number);
-
-	gtk_widget_show_all(container);
-
-	control = bonobo_control_new(container);
-
-	/* Property Bag */
-	property_bag = bonobo_property_bag_new(property_control_get_prop,
-										   NULL, control);
-
-	bonobo_property_bag_add(property_bag, "bonobo:title",
-							PROP_CONTROL_TITLE, BONOBO_ARG_STRING,
-							NULL, NULL, BONOBO_PROPERTY_READABLE);
-
-	bonobo_object_add_interface(BONOBO_OBJECT(control),
-								BONOBO_OBJECT(property_bag));
-
-	return control;
-#else
-	return NULL;
-#endif
-}
-
-static void
-document_changed_cb(GtkGS *gs, gpointer data)
-{
-	GgvPostScriptView *ps_view = GGV_POSTSCRIPT_VIEW(data);
-
-	gtk_gs_reload(gs);
-	notify_page_count_change(GGV_POSTSCRIPT_VIEW(data));
-	notify_page_change(GGV_POSTSCRIPT_VIEW(data));
-	notify_status_change(GGV_POSTSCRIPT_VIEW(data));
+    static GType type = 0;
+    
+    if (type == 0) {
+		static GTypeInfo info = {
+			sizeof (GgvPostScriptViewClass),
+			(GBaseInitFunc) NULL,
+			(GBaseFinalizeFunc) NULL,
+			(GClassInitFunc) ggv_postscript_view_class_init,
+			NULL, NULL,
+			sizeof (GgvPostScriptView),
+			0,
+			(GInstanceInitFunc) ggv_postscript_view_init
+		};
+		
+		static const GInterfaceInfo ggv_document_info = {
+			(GInterfaceInitFunc) ggv_document_interface_init,
+			NULL,
+			NULL
+		};
+		
+		type = bonobo_type_unique(BONOBO_TYPE_OBJECT,
+								  POA_GNOME_GGV_PostScriptView__init,
+								  POA_GNOME_GGV_PostScriptView__fini,
+								  G_STRUCT_OFFSET (GgvPostScriptViewClass, epv),
+								  &info, "GgvPostScriptView");
+		
+		g_type_add_interface_static (type,
+									 GGV_DOCUMENT_TYPE,
+									 &ggv_document_info);
+    }
+    return type;
 }
 
 static void
@@ -1812,7 +1826,8 @@ interpreter_message_cb(GtkGS *gs, gchar *msg, gpointer data)
 	GgvPostScriptView *ps_view = GGV_POSTSCRIPT_VIEW(data);
 
 #if 0
-	/* actually, these are quite useless, so we won't show them... */
+	/* actually, these are quite useless, so we don't want to show them,
+	   hence the '#if 0'... */
 	if(ps_view->priv->msg_win)
 		ggv_msg_window_add_text(ps_view->priv->msg_win, msg, TRUE);
 #endif
@@ -1834,11 +1849,12 @@ interpreter_error_cb(GtkGS *gs, gint status, gpointer data)
 static void
 view_realized_cb(GtkGS *gs, gpointer data)
 {
-	GdkCursor *cursor;
+	GgvPostScriptView *ps_view = GGV_POSTSCRIPT_VIEW(data);
 
-	cursor = cursor_get(GTK_WIDGET(gs)->window, CURSOR_HAND_OPEN);
-	gdk_window_set_cursor(GTK_WIDGET(gs)->window, cursor);
-	gdk_cursor_unref(cursor);
+	if(ps_view->priv->win_cursor == NULL)
+		ps_view->priv->win_cursor =
+			cursor_get(GTK_WIDGET(gs)->window, CURSOR_HAND_OPEN);
+	gdk_window_set_cursor(GTK_WIDGET(gs)->window, ps_view->priv->win_cursor);
 }
 
 static void
@@ -1859,11 +1875,6 @@ ggv_postscript_view_prefs_changed(GConfClient *client, guint cnxn_id,
 		gtk_gs_set_default_orientation(gs, gconf_value_get_int(entry->value));
 		sync_orientation_items(ps_view);
 	}
-#if 0
-	else if(!strcmp(entry->key, "/apps/ggv/gtkgs/zoom")) {
-		gtk_gs_set_zoom(gs, gconf_value_get_float(entry->value));
-	}
-#endif
 	else if(!strcmp(entry->key, "/apps/ggv/gtkgs/size")) {
 		gtk_gs_set_default_size(gs, gconf_value_get_int(entry->value));
 		sync_size_items(ps_view);
@@ -1874,8 +1885,6 @@ ggv_postscript_view_prefs_changed(GConfClient *client, guint cnxn_id,
 		gtk_gs_set_override_size(gs, gconf_value_get_bool(entry->value));
 		sync_size_items(ps_view);
 	}
-	else if(!strcmp(entry->key, "/apps/ggv/gtkgs/watch_doc"))
-		gtk_gs_set_watch_file(gs, gconf_value_get_bool(entry->value));
 	else if(!strcmp(entry->key, "/apps/ggv/gtkgs/scrollstep"))
 		gtk_gs_set_scroll_step(gs, gconf_value_get_float(entry->value));
 	else if(!strcmp(entry->key, "/apps/ggv/gtkgs/show_scroll_rect"))
@@ -1884,6 +1893,8 @@ ggv_postscript_view_prefs_changed(GConfClient *client, guint cnxn_id,
 	   ps_view->priv->pane_auto_jump = gconf_value_get_bool(entry->value);
 	else if(!strcmp(entry->key, "/apps/ggv/control/pageflip"))
 	   ps_view->priv->page_flip = gconf_value_get_bool(entry->value);
+	/* note that we don't want to handle changes to some parameters
+	   that are considered "default": zoom, etc. */
 }
 
 static gboolean
@@ -1926,6 +1937,13 @@ sidebar_key_press_event(GtkWidget *widget, GdkEventKey *event,
 	return TRUE;
 }
 
+static void
+sidebar_set_page(GgvSidebar *sidebar, int page, gpointer data)
+{
+	GgvPostScriptView *ps_view = GGV_POSTSCRIPT_VIEW(data);
+	ggv_postscript_view_goto_page(ps_view, page);
+}
+
 GgvPostScriptView *
 ggv_postscript_view_construct(GgvPostScriptView *ps_view,
 							  GtkGS *gs, gboolean zoom_fit)
@@ -1965,10 +1983,8 @@ ggv_postscript_view_construct(GgvPostScriptView *ps_view,
 					 G_CALLBACK(interpreter_message_cb), ps_view);
 	g_signal_connect(G_OBJECT(gs), "interpreter_error",
 					 G_CALLBACK(interpreter_error_cb), ps_view);
-	g_signal_connect(G_OBJECT(gs), "document_changed",
-					 G_CALLBACK(document_changed_cb), ps_view);
 	g_signal_connect(G_OBJECT(gs), "realize",
-					 G_CALLBACK(view_realized_cb), NULL);
+					 G_CALLBACK(view_realized_cb), ps_view);
 
 	gtk_widget_show(GTK_WIDGET(gs));
 
@@ -2032,25 +2048,19 @@ ggv_postscript_view_construct(GgvPostScriptView *ps_view,
 							PROP_RESPECT_EOF, TC_CORBA_boolean, NULL,
 							_("Respect EOF comment"), 
 							BONOBO_PROPERTY_READABLE | BONOBO_PROPERTY_WRITEABLE);
-	bonobo_property_bag_add(ps_view->priv->property_bag, "watch_file",
-							PROP_WATCH_FILE, TC_CORBA_boolean, NULL,
-							_("Watch displayed file for changes"), 
-							BONOBO_PROPERTY_READABLE | BONOBO_PROPERTY_WRITEABLE);
 	bonobo_property_bag_add(ps_view->priv->property_bag, "antialiasing",
 							PROP_ANTIALIASING, TC_CORBA_boolean, NULL,
 							_("Antialiasing"), 
 							BONOBO_PROPERTY_READABLE | BONOBO_PROPERTY_WRITEABLE);
 
-	/* Property Control */
-	ps_view->priv->property_control = bonobo_property_control_new
-		(property_control_get_cb, 1, ps_view);
-
 	/* UI Component */
 	ps_view->priv->uic = bonobo_ui_component_new("GgvPostScriptView");
 
-	ps_view->priv->sidebar = ggv_sidebar_new(ps_view);
+	ps_view->priv->sidebar = ggv_sidebar_new(GGV_DOCUMENT(ps_view));
 	g_signal_connect(G_OBJECT(ggv_sidebar_get_checklist(ps_view->priv->sidebar)),
 					 "key_press_event", G_CALLBACK(sidebar_key_press_event), ps_view);
+	g_signal_connect(G_OBJECT(ps_view->priv->sidebar), "set_page",
+					 G_CALLBACK(sidebar_set_page), ps_view);
 
 	ps_view->priv->pane_auto_jump =
 		gconf_client_get_bool(gtk_gs_defaults_gconf_client(),
