@@ -15,9 +15,6 @@
 #include <bonobo.h>
 #include <bonobo/bonobo-ui-main.h>
 #include <libgnomevfs/gnome-vfs-utils.h>
-#include <X11/Xlib.h>
-#include <X11/Xatom.h> 
-#include <gdk/gdkx.h>
 
 #include <math.h>
 #include <ctype.h>
@@ -26,7 +23,6 @@
 #include "gtkchecklist.h"
 #include "ggv-prefs.h"
 #include "ggv-window.h"
-#include "ggv-file-sel.h"
 #include "ggvutils.h"
 #include "ggv-recent.h"
 #include "recent-files/egg-recent-view.h"
@@ -59,11 +55,6 @@ set_file_cmds_sensitivity(GgvWindow *win, gboolean sens)
                                      "sensitive",
                                      sens?"1":"0",
                                      NULL);
-        bonobo_ui_component_set_prop(win->uic,
-                                     "/commands/FileClose",
-                                     "sensitive",
-                                     sens?"1":"0",
-                                     NULL);
 }
 
 static void
@@ -144,14 +135,6 @@ ggv_window_remove_control(GgvWindow *win)
                 bonobo_object_release_unref(win->pb, NULL);
                 win->pb = CORBA_OBJECT_NIL;
         }
-#if 0
-        if(win->view) {
-                gtk_container_remove(GTK_CONTAINER(win->hbox),
-                                     win->view);
-                gtk_widget_unref(win->view);
-                win->view = NULL;
-        }
-#endif
         if(win->filename) {
                 g_free(win->filename);
                 win->filename = NULL;
@@ -164,52 +147,6 @@ ggv_window_remove_control(GgvWindow *win)
         CORBA_exception_free(&ev);                
 }
 
-/* utility functions to raise window above all others! */
-#define XA_WIN_LAYER         "_WIN_LAYER"
-typedef enum {
-        WIN_LAYER_NORMAL = 4,
-        WIN_LAYER_ABOVE_DOCK = 10
-} GgvWindowLayer;
-static Atom A_XA_WIN_LAYER;
-static gboolean atom_initialized = FALSE;
-
-static void
-ggv_init_atoms()
-{
-        if(!GDK_DISPLAY() || atom_initialized) return;
-        A_XA_WIN_LAYER = XInternAtom(GDK_DISPLAY(), XA_WIN_LAYER, False);
-        atom_initialized = TRUE;
-}
-
-static void
-ggv_window_set_layer(GgvWindow *win, GgvWindowLayer layer)
-{
-        if(!atom_initialized)
-                ggv_init_atoms();
-        if(GTK_WIDGET_MAPPED(win)) {
-                XEvent e;
-                e.type = ClientMessage;
-                e.xclient.type = ClientMessage;
-                e.xclient.window = GDK_WINDOW_XID(GTK_WIDGET(win)->window);
-                e.xclient.message_type = A_XA_WIN_LAYER;
-                e.xclient.format = 32;
-                e.xclient.data.l[0] = (long)layer;
-                e.xclient.data.l[1] = GDK_CURRENT_TIME;
-
-                XSendEvent(GDK_DISPLAY(), GDK_ROOT_WINDOW(), False,
-                           SubstructureNotifyMask, (XEvent *)&e);
-        }
-        else {
-                long data[1];
-
-                data[0] = layer;
-                XChangeProperty(GDK_DISPLAY(),
-                                GDK_WINDOW_XID(GTK_WIDGET(win)->window),
-                                A_XA_WIN_LAYER, XA_CARDINAL, 32,
-                                PropModeReplace, (unsigned char *)data, 1);
-        }
-}
-
 static void
 ggv_window_set_fullscreen(GgvWindow *win, gboolean fs)
 {
@@ -250,6 +187,7 @@ ggv_window_set_fullscreen(GgvWindow *win, gboolean fs)
                 gtk_window_resize(GTK_WINDOW(win),
                                   gdk_screen_width(),
                                   gdk_screen_height());
+		gtk_window_fullscreen (GTK_WINDOW(win));
         }
         else {
                 win->show_menus = win->orig_sm;
@@ -273,8 +211,8 @@ ggv_window_set_fullscreen(GgvWindow *win, gboolean fs)
                 }
                 gtk_window_move(GTK_WINDOW(win), win->orig_x, win->orig_y);
                 gtk_window_resize(GTK_WINDOW(win), win->orig_width, win->orig_height);
+		gtk_window_unfullscreen (GTK_WINDOW(win));
         }
-        ggv_window_set_layer(win, fs?WIN_LAYER_ABOVE_DOCK:WIN_LAYER_NORMAL);
         sync_settings_menu_items(win);
         sync_settings_popup_items(win);
         sync_fullscreen_items(win);
@@ -311,7 +249,7 @@ ggv_window_drag_data_received(GtkWidget *widget,
                 GtkWidget *newwin;
                 gchar **uri = win->uris_to_open;
 
-                if(win->current_page < 0)
+                if(win->page_count <= 0)
                         newwin = GTK_WIDGET(win);
                 else
                         newwin = NULL;
@@ -348,40 +286,16 @@ control_property_changed_handler(BonoboListener    *listener,
                                         BONOBO_ARG_GET_STRING(any));
         }
         else if(!g_ascii_strcasecmp(event_name, "Bonobo/Property:change:page")) {
-                if(win->current_page == BONOBO_ARG_GET_LONG(any) &&
-                   win->current_page < 0)
-                        return;
-
-                win->current_page = BONOBO_ARG_GET_LONG(any);
-                if(win->current_page < 0) {
-                        GtkWidget *dlg;
-                        BonoboArg *arg;
-                        gchar *msg;
-
-                        gtk_window_set_title(GTK_WINDOW(win),
-                                             _("GGV: no document loaded"));
-
-                        arg = bonobo_pbclient_get_value(win->pb, "status",
-                                                        NULL, NULL);
-                        if(arg == NULL)
-                                return;
-                        msg = BONOBO_ARG_GET_STRING(arg);
-                        if(*msg != '\0') {
-                                dlg = gtk_message_dialog_new(GTK_WINDOW(win),
-                                                             GTK_DIALOG_MODAL,
-                                                             GTK_MESSAGE_ERROR,
-                                                             GTK_BUTTONS_OK,
-                                                             msg);
-                                gtk_widget_show(dlg);
-                                gtk_dialog_run(GTK_DIALOG(dlg));
-                                gtk_widget_destroy(dlg);
-                        }
-                        bonobo_arg_release(arg);
-                        return;
-                }
+                if(win->current_page != BONOBO_ARG_GET_LONG(any))
+                        win->current_page = BONOBO_ARG_GET_LONG(any);
         }
         else if(!g_ascii_strcasecmp(event_name, "Bonobo/Property:change:page_count")) {
-                set_file_cmds_sensitivity(win, (BONOBO_ARG_GET_LONG(any) > 0));
+                win->page_count = BONOBO_ARG_GET_LONG(any);
+                set_file_cmds_sensitivity(win, win->page_count > 0);
+                if(win->page_count <= 0) {
+                        gtk_window_set_title(GTK_WINDOW(win),
+                                             _("GGV: no document loaded"));
+                }
         }
 }
 
@@ -392,11 +306,38 @@ verb_FileOpen(BonoboUIComponent *uic, gpointer data, const char *cname)
         EggRecentModel *recent;
 
         GgvWindow *win = GGV_WINDOW(data);
-        gchar *fname, *escaped_fname;
+        gchar *fname = NULL, *escaped_fname;
+	GtkWidget *chooser;
+
+	chooser = gtk_file_chooser_dialog_new(_("Select a PostScript document"),
+					      GTK_WINDOW(win),
+					      GTK_FILE_CHOOSER_ACTION_OPEN,
+					      GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					      GTK_STOCK_OPEN, GTK_RESPONSE_OK,
+					      NULL);
+	gtk_dialog_set_default_response(GTK_DIALOG(chooser), GTK_RESPONSE_OK);
+	gtk_window_set_default_size(GTK_WINDOW(chooser), 600, 400);
+	if (win->filename)
+		gtk_file_chooser_set_uri(GTK_FILE_CHOOSER(chooser),
+					 win->filename);
+
+	if (gtk_dialog_run(GTK_DIALOG(chooser)) == GTK_RESPONSE_OK)
+		fname = gtk_file_chooser_get_uri(GTK_FILE_CHOOSER(chooser));
+
+	gtk_widget_destroy(chooser);
 
-        fname = ggv_file_sel_request_uri(_("Select a PostScript document"),
-                                         win->filename);
         if(fname) {
+                if(win->page_count > 0) {
+                        GtkWidget *newwin;
+
+                        newwin = ggv_window_new();
+                        if(NULL == newwin) {
+                                g_free(fname);
+                                return;
+                        }
+                        gtk_widget_show(newwin);
+                        win = GGV_WINDOW(newwin);
+                }
                 if(!ggv_window_load(win, fname)) {
                         GtkWidget *dlg;
                         dlg = gtk_message_dialog_new(GTK_WINDOW(win),
@@ -411,7 +352,6 @@ verb_FileOpen(BonoboUIComponent *uic, gpointer data, const char *cname)
                 }
 		recent = ggv_recent_get_model ();
 		item = egg_recent_item_new_from_uri (fname);
-		egg_recent_item_add_group (item, "ggv");
 		egg_recent_model_add_full (recent, item);
 		egg_recent_item_unref (item);
 
@@ -423,21 +363,8 @@ static void
 verb_FileReload(BonoboUIComponent *uic, gpointer data, const char *cname)
 {
         GgvWindow *win = GGV_WINDOW(data);
-        GNOME_GGV_PostScriptView ps_view;
-        CORBA_Environment ev;
 
-        CORBA_exception_init(&ev);
-        ps_view = (GNOME_GGV_PostScriptView)
-                Bonobo_Unknown_queryInterface(win->control,
-                                              "IDL:GNOME/GGV/PostScriptView:1.0",
-                                              &ev);
-        if(!BONOBO_EX(&ev)) {
-                GNOME_GGV_PostScriptView_reload(ps_view, &ev);
-                if(BONOBO_EX(&ev))
-                        g_warning("Could not invoke reload method.");
-                bonobo_object_release_unref(ps_view, &ev);
-        }
-        CORBA_exception_free(&ev);
+        ggv_window_reload(win);
 }
 
 static void
@@ -445,49 +372,16 @@ verb_FileClose(BonoboUIComponent *uic, gpointer data, const char *cname)
 {
         GgvWindow *win = GGV_WINDOW(data);
 
-        if(window_list->next != NULL)
-                ggv_window_close(win);
-        else {
-                GNOME_GGV_PostScriptView ps_view;
-                CORBA_Environment ev;
-
-                CORBA_exception_init(&ev);
-                ps_view = (GNOME_GGV_PostScriptView)
-                        Bonobo_Unknown_queryInterface(win->control,
-                                                      "IDL:GNOME/GGV/PostScriptView:1.0",
-                                                      &ev);
-                if(!BONOBO_EX(&ev)) {
-                        GNOME_GGV_PostScriptView_close(ps_view, &ev);
-                        if(BONOBO_EX(&ev))
-                                g_warning("Could not invoke reload method.");
-                        bonobo_object_release_unref(ps_view, &ev);
-                }
-                CORBA_exception_free(&ev);                
-                ggv_window_set_watch_file(win, FALSE);
-        }
-}
-
-static void
-verb_FileNew(BonoboUIComponent *uic, gpointer data, const char *cname)
-{
-        GtkWidget *win = ggv_window_new();
-        gtk_widget_show(win);
+        ggv_window_close(win);
+        if (window_list == NULL)
+                bonobo_main_quit();
 }
 
 void
 ggv_window_destroy_all()
 {
         while(window_list)
-                gtk_widget_destroy(GTK_WIDGET(window_list->data));
-}
-
-static void
-verb_FileExit(BonoboUIComponent *uic, gpointer data, const char *cname)
-{
-        GList *l;
-
-        ggv_window_destroy_all();
-        bonobo_main_quit();
+                ggv_window_close(GGV_WINDOW(window_list->data));
 }
 
 static void
@@ -614,7 +508,7 @@ verb_HelpAbout(BonoboUIComponent *uic, gpointer data, const char *cname)
 		about = gnome_about_new (
 			_("Gnome Ghostview"),
 			VERSION,
-			"Copyright \xc2\xa9 1998-2002 Free Software Foundation, Inc.",
+			"Copyright \xc2\xa9 1998-2004 Free Software Foundation, Inc.",
 			_("The GNOME PostScript document previewer"),
 			authors,
 			documenters,
@@ -635,6 +529,10 @@ static void
 verb_HelpContents(BonoboUIComponent *uic, gpointer data, const char *cname)
 {
 	GError *error = NULL;
+        GgvWindow *win = GGV_WINDOW(data);
+
+        if(win->fullscreen)
+                ggv_window_set_fullscreen(win, FALSE);
 	gnome_help_display ("ggv",NULL,&error);
 }
 
@@ -694,11 +592,9 @@ verb_DnDCancel(BonoboUIComponent *uic, gpointer user_data, const char *cname)
 
 /* our verb list */
 static BonoboUIVerb ggv_app_verbs[] = {
-        BONOBO_UI_VERB("FileNew", verb_FileNew),
         BONOBO_UI_VERB("FileClose", verb_FileClose),
         BONOBO_UI_VERB("FileOpen", verb_FileOpen),
         BONOBO_UI_VERB("FileReload", verb_FileReload),
-        BONOBO_UI_VERB("FileExit", verb_FileExit),
         BONOBO_UI_VERB("HelpAbout", verb_HelpAbout),
         BONOBO_UI_VERB("Help", verb_HelpContents),
         BONOBO_UI_VERB("DnDNewWindow", verb_DnDNewWindow),
@@ -946,7 +842,7 @@ ggv_file_open_recent (EggRecentView *view, EggRecentItem *item, gpointer data)
                                                                                                                              
         uri_utf8 = egg_recent_item_get_uri_utf8 (item);
                                                                                                                              
-        if(win->current_page < 0)
+        if(win->page_count <= 0)
                 newwin = GTK_WIDGET (win);
         else
                 newwin = NULL;
@@ -1032,7 +928,7 @@ ggv_window_new(void)
 	win = GGV_WINDOW(g_object_new(GGV_TYPE_WINDOW, "win_name", "ggv", "title", _("GGV: no document loaded"), NULL));
         win->control = control;
 
-        win->current_page = -2;
+        win->current_page = 0;
         win->show_toolbar = ggv_toolbar;
         win->show_menus = ggv_menubar;
         win->show_statusbar = ggv_statusbar;
@@ -1129,16 +1025,6 @@ ggv_window_new(void)
                         "Bonobo/Property:change:height,"
                         "Bonobo/Property:change:title,"
                         "Bonobo/Property:change:status";
-                bonobo_ui_component_set_prop(win->uic,
-                                             "/commands/FileReload",
-                                             "sensitive",
-                                             (bonobo_pbclient_get_long(pb, "page", NULL) >= 0)?"1":"0",
-                                             NULL);
-                bonobo_ui_component_set_prop(win->uic,
-                                             "/commands/FileClose",
-                                             "sensitive",
-                                             (bonobo_pbclient_get_long(pb, "page", NULL) >= 0)?"1":"0",
-                                             NULL);
                 bonobo_event_source_client_add_listener(pb,
                                                         (BonoboListenerCallbackFn)control_property_changed_handler,
                                                         mask, NULL, win);
@@ -1177,6 +1063,8 @@ ggv_window_new(void)
                                 (GConfClientNotifyFunc)prefs_changed,
                                 win, NULL, NULL);
 
+        set_file_cmds_sensitivity(win, FALSE);
+
 	return GTK_WIDGET(win);
 }
 
@@ -1185,7 +1073,7 @@ ggv_window_load(GgvWindow *win, const gchar *filename)
 {
         CORBA_Environment ev;
         Bonobo_PersistFile pf;
-        gchar *title, *unescaped, *utf8_title;
+        gchar *title, *unescaped, *utf8_title, *_filename;
         GnomeVFSFileInfo info;
 
         g_return_val_if_fail(win != NULL, FALSE);
@@ -1213,9 +1101,10 @@ ggv_window_load(GgvWindow *win, const gchar *filename)
         Bonobo_PersistFile_load(pf, filename, &ev);
         bonobo_object_release_unref(pf, NULL);
 
+        _filename = g_strdup(filename);
         if(win->filename)
                 g_free(win->filename);
-        win->filename = g_strdup(filename);
+        win->filename = _filename;
 
         unescaped = gnome_vfs_unescape_string_for_display(win->filename);
         title = g_strconcat(_("GGV: "), unescaped, NULL);
@@ -1233,11 +1122,7 @@ ggv_window_load(GgvWindow *win, const gchar *filename)
 
         bonobo_ui_component_set_prop
                 (win->uic, "/commands/FileReload", "sensitive",
-                 (bonobo_pbclient_get_long(win->pb, "page", NULL) >= 0)?"1":"0",
-                 NULL);
-        bonobo_ui_component_set_prop
-                (win->uic, "/commands/FileClose", "sensitive",
-                 (bonobo_pbclient_get_long(win->pb, "page", NULL) >= 0)?"1":"0",
+                 (bonobo_pbclient_get_long(win->pb, "page_count", NULL) > 0)?"1":"0",
                  NULL);
 
         ggv_window_set_watch_file(win, ggv_watch_doc);
@@ -1251,6 +1136,8 @@ ggv_window_close(GgvWindow *win)
         g_return_if_fail(win != NULL);
         g_return_if_fail(GGV_IS_WINDOW(win));
 
+        bonobo_control_frame_control_deactivate(win->ctlframe);
+
         gtk_widget_destroy(GTK_WIDGET(win));
 
         if(!window_list)
@@ -1263,6 +1150,14 @@ ggv_get_window_list()
         return window_list;
 }
 
+void
+ggv_window_reload(GgvWindow *win)
+{
+        gint page = win->current_page;  
+        ggv_window_load(win, win->filename);
+        bonobo_pbclient_set_long(win->pb, "page", page, NULL);
+}
+
 /* Reload file if it was changed */
 static gint
 timer_callback (gpointer data)
@@ -1286,25 +1181,10 @@ timer_callback (gpointer data)
                 return TRUE;
 
         /* reload file if necessary */
-	if ((win->current_page >= 0) &&
-	    (info.mtime != win->mtime) &&
+	if ((info.mtime != win->mtime) &&
 	    (time (NULL) - info.mtime >= GGV_WATCH_TIMEOUT) &&
             (NULL != win->control)) {
-                GNOME_GGV_PostScriptView ps_view;
-                CORBA_Environment ev;
-
-                CORBA_exception_init(&ev);
-                ps_view = (GNOME_GGV_PostScriptView)
-                        Bonobo_Unknown_queryInterface(win->control,
-                                                      "IDL:GNOME/GGV/PostScriptView:1.0",
-                                                      &ev);
-                if(!BONOBO_EX(&ev)) {
-                        GNOME_GGV_PostScriptView_reload(ps_view, &ev);
-                        if(BONOBO_EX(&ev))
-                                g_warning("Could not invoke reload method.");
-                        bonobo_object_release_unref(ps_view, &ev);
-                }
-                CORBA_exception_free(&ev);
+                ggv_window_reload(win);
                 win->mtime = info.mtime;
         }
 
